(
	--example tool to copy/paste and save/load layersets across sessions
	
	--Globals
	global VRayVFB2layerSetRoll
	try (destroyDialog VRayVFB2layerSetRoll) catch()
	global vrayVFB2JSONString --max session-persistent global variable for the JSON string
	
	--Locals
	local vrayVFB2mgr = undefined
	local vfblFilePath = ""
	
	--functions
	fn validateIncomingJSONString JSONString =
	(
		returnValue = false
		patterns=#("{\"Version\":*", "*\"class\":\"root\"*", "*\"class\":\"chaos.displayCorrection\"*", "*\"class\":\"chaos.ref.sourcefolder\"*") --Any of the four patterns alone would do as they all, always, appear in a copied JSON string. I'm listing, and checking against, them all for convenience.
		if JSONString != "" and JSONString != undefined do
		(
			for p in patterns do
			(
				returnValue = (matchPattern JSONString pattern:p)  --returns true or false
				if returnValue== false do exit with returnValue
			)
		)
		returnValue
	)
	
	fn toggleRolloutButtons = --making changes only if the state is different prevents flickering in the UI
	(
		oldState = VRayVFB2layerSetRoll.btnPst.enabled
		if ( validateIncomingJSONString vrayVFB2JSONString ) then 
			( 
				if oldState == false do VRayVFB2layerSetRoll.btnPst.enabled = true
			)
		else
			(
				if oldState == true do VRayVFB2layerSetRoll.btnPst.enabled = false
			)
		
		oldState = VRayVFB2layerSetRoll.btnOsP.enabled
		if ( validateIncomingJSONString (getclipboardText()) ) then 
			( 
				if oldState == false do VRayVFB2layerSetRoll.btnOsP.enabled = true
			)
		else
			(
				if oldState == true do VRayVFB2layerSetRoll.btnOsP.enabled = false
			)
	)
	
	rollout VRayVFB2layerSetRoll "Sample VFB2 Maxscript Sample Tool" width:296 height:296
	(
		button 'btnCpy' "Copy Current Layerset to Memory" pos:[8,8] width:280 height:40 align:#left tooltip:"Copies the current layerset to memory. Buffer will wipe on Max close, not on scene load or reset."
		button 'btnPst' "Paste Current Layerset from Memory" pos:[8,56] width:280 height:40 enabled:false align:#left tooltip:"Paste from the Max Memory Buffer. If the button is gray, no valid copy buffer has been detected. Copy again."
		button 'btnOsC' "Copy Current Layerset to OS Clipboard" pos:[8,104] width:280 height:40 align:#left tooltip:"Copies the current layerset to the OS' Copy Buffer. Buffer will survive Max closing, and operations, but can be overwritten by other CTRL+C operations even outside of Max."
		button 'btnOsP' "Paste Current Layerset from OS Clipboard" pos:[8,152] width:280 height:40 enabled:false align:#left tooltip:"Paste from the OS' Copy Buffer. If the button is gray, no valid copy buffer has been detected. Copy again."
		button 'btnSv' "Save Current Layerset to File" pos:[8,200] width:280 height:40 align:#left tooltip:"Save the current Layer Set to a file on disk"
		button 'btnLd' "Load Current Layerset from File" pos:[8,248] width:280 height:40 align:#left tooltip:"Load a layer Set from a file on disk"
		
		timer 'tmr1' "Timer" pos:[288,290] width:24 height:24 interval:100 align:#left
		
		on tmr1 tick do toggleRolloutButtons()
			
		on btnCpy pressed do
		(
			vrayVFB2mgr = (vfbcontrol #getlayermgr)[1]  --gets the manager, which is returned in a single-element array. i take the first element immediately, so to access the manager directly
			if vrayVFB2mgr != undefined do --we did get a valid manager
			(
				vrayVFB2JSONString = vrayVFB2mgr.saveLayersToJSON() --copy the layerset to a JSON string
			)
		)
		
		on btnPst pressed do
		(
			vrayVFB2mgr = (vfbcontrol #getlayermgr)[1]  --gets the manager, which is returned in a single-element array. i take the first element immediately, so to access the manager directly
			if vrayVFB2mgr != undefined and (validateIncomingJSONString vrayVFB2JSONString) do --we did get a valid manager and the string to paste isn't empty
			(
				vrayVFB2mgr.loadLayersFromJSON vrayVFB2JSONString
			)				
		)
		
		on btnOsC pressed do
		(
			vrayVFB2mgr = (vfbcontrol #getlayermgr)[1]  --gets the manager, which is returned in a single-element array. i take the first element immediately, so to access the manager directly
			if vrayVFB2mgr != undefined do --we did get a valid manager
			(
				setclipboardText (vrayVFB2mgr.saveLayersToJSON()) --copy the layerset to a JSON string and store it in the OS copy/paste clipboard
			)
		)
		on btnOsP pressed do
		(
			vrayVFB2mgr = (vfbcontrol #getlayermgr)[1]  --gets the manager, which is returned in a single-element array. i take the first element immediately, so to access the manager directly
			if vrayVFB2mgr != undefined do --we did get a valid manager and the string to paste isn't empty
			(
				pasteString = (getclipboardText())
				if (validateIncomingJSONString pasteString) do --early safety check to avoid pasting something else
				(
					vrayVFB2mgr.loadLayersFromJSON pasteString --Paste the layerset from a valid OS copy/paste clipboard
				)
			)				
		)
		
		on btnSv pressed do
		(
			vrayVFB2mgr = (vfbcontrol #getlayermgr)[1]  --gets the manager, which is returned in a single-element array. i take the first element immediately, so to access the manager directly
			if vrayVFB2mgr != undefined do --we did get a valid manager
			(
				vfblFilePath=getSaveFileName caption:"Choose a path and filename to save layers to" filename:"*.vfbl" types:"VFB2 Layerset Files | *.vfbl" historyCategory:"VFB2LayerFiles" --get a valid filename and path
				if vfblFilePath != "" do  --if path is not empty
				(
					if getFilenameType vfblFilePath != ".vfbl" do vfblFilePath+=".vfbl" --ensure the extension is present, even if the user hasn't typed it
					vrayVFB2mgr.saveLayersToFile vfblFilePath --save the layerset to a VFBL file
				)
			)
		)
		on btnLd pressed do
		(
			vrayVFB2mgr = (vfbcontrol #getlayermgr)[1]  --gets the manager, which is returned in a single-element array. i take the first element immediately, so to access the manager directly
			if vrayVFB2mgr != undefined do --we did get a valid manager
			(
				vfblFilePath=getOpenFileName caption:"Choose file to load layers from" filename:"*.vfbl" types:"VFB2 Layerset Files | *.vfbl" historyCategory:"VFB2LayerFiles" --get a valid filename and path
				if vfblFilePath != "" do  --if path is not empty
				(
					vrayVFB2mgr.loadLayersFromFile vfblFilePath --save the layerset to a VFBL file
				)
			)
		)
	)
	
	createDialog VRayVFB2layerSetRoll
)

